class Problem:
    def __init__(self, initial, goal=None):
        # 指定搜索问题的初始状态和目标状态
        self.initial = initial
        self.goal = goal

    def actions(self, state):
        # 返回在给定状态下可以采取的动作
        raise NotImplementedError

    def transition(self, state, action):
        # 执行给定动作后达到的状态
        raise NotImplementedError

    def goal_test(self, state):
        # 给定状态是目标状态则返回True
        return state == self.goal


class Node:
    def __init__(self, state, parent=None, action=None):
        # 构造节点，记录当前状态、父节点和采取的动作
        self.state = state
        self.parent = parent
        self.action = action

    def __eq__(self, other):
        return isinstance(other, Node) and self.state == other.state

    def expand(self, problem: Problem):
        # 展开当前问题可以到达的所有子节点
        child = []
        for action in problem.actions(self.state):
            next_state = problem.transition(self.state, action)
            child.append(Node(next_state, self, action))
        return child

    def path(self):
        # 返回从根节点到该节点的路径
        if self.parent is None:
            return [self]
        return self.parent.path() + [self]


class RiverCrossing(Problem):
    def __init__(self, forbidden_states=None):
        super().__init__(initial=(1, 1, 1, 1), goal=(-1, -1, -1, -1))
        if forbidden_states is None:
            forbidden_states = set()
        self.forbidden_states = set(forbidden_states)

    def actions(self, state):
        if state in self.forbidden_states:
            return []
        allowed_actions = [(-1, 1, 1, 1)]
        for n in range(1, len(state)):
            if state[n] == state[0]:
                action = tuple([-1 if i in [0, n] else 1 for i in range(4)])
                allowed_actions.append(action)

        return allowed_actions

    def transition(self, state, action):
        return tuple(s * a for s, a in zip(state, action))


def TreeSearch(problem: Problem):
    # 实现树搜索算法
    frontier = [Node(problem.initial)]
    explored = set()
    while frontier:
        node = frontier.pop()
        if problem.goal_test(node.state):
            return node
        explored.add(node.state)
        frontier.extend(n for n in node.expand(problem) if n.state not in explored)
    return None


def riverCrossing():
    forbidden_states = {
        (1, 1, -1, -1),
        (1, -1, -1, 1),
        (1, -1, -1, -1),
        (-1, -1, 1, 1),
        (-1, 1, 1, -1),
        (-1, 1, 1, 1)
    }
    river_crossing = RiverCrossing(forbidden_states=forbidden_states)
    result = TreeSearch(river_crossing)
    if result:
        for node in result.path():
            print("".join(["N" if s == 1 else "F" for s in node.state]))
    else:
        print("未找到可行路径")


if __name__ == "__main__":
    riverCrossing()



class Game:
    def __init__(self, initial, players=()):
        # 创建游戏初始状态，记录两个玩家信息
        self.initial = initial
        self.players = players

    def opponent(self, player):
        # 返回当前玩家的对手玩家
        return self.players[(self.players.index(player) + 1) % 2]

    def moves(self, state):
        # 返回在给定状态下可以采取的行动
        raise NotImplementedError

    def transition(self, state, move):
        # 返回玩家执行给定行动后到达的游戏状态
        raise NotImplementedError

    def terminal_test(self, state):
        # 判断游戏状态是否为终结状态
        return not self.moves(state=state)

    def utility(self, state, player):
        # 返回给定状态对于给定玩家的效用值
        raise NotImplementedError


class GameState:
    def __init__(self, board, to_move, score=None):
        self.board = board
        self.to_move = to_move
        self.score = score

    def __repr__(self):
        return self.board.__repr__()


class Board(dict):
    def __init__(self, width, height):
        super().__init__()
        self.width = width
        self.height = height
        self.squares = {(x, y) for x in range(1, width + 1)
                        for y in range(1, height + 1)}

    def __repr__(self):
        # 以字符串形式显示棋盘状态
        rows = []
        for y in range(1, self.height + 1):
            row = []
            for x in range(1, self.width + 1):
                row.append(self.get((x, y), "·"))
            rows.append(" ".join(row))
        return "\n".join(rows)

    def blank_squares(self):
        # 返回可以落子的地方
        return self.squares - set(self)

    def new(self):
        # 新建空的棋盘实例， 复制原来棋盘的棋子状态信息
        new_board = self.__class__(width=self.width, height=self.height)
        new_board.update(self)
        return new_board

    def k_in_line(self, k, player, move):
        # 如果玩家落子后连成一条长度为k的线则返回True
        def in_line(move, delta):
            (delta_x, delta_y) = delta
            x, y = move
            n = 0
            while self.get((x, y)) == player:
                n += 1
                x, y = x + delta_x, y + delta_y
            x, y = move
            while self.get((x, y)) == player:
                n += 1
                x, y = x - delta_x, y - delta_y
            n -= 1
            return n >= k

        return any(in_line(move, delta)
                   for delta in ((0, 1), (1, 0), (1, 1), (1, -1)))



from arcade import color, key
from arcade import *
import random

WIDTH = 800
HEIGHT = 600
TITLE = "Hit The Block"
RECT_WIDTH = 100
RECT_HEIGHT = 10
BLOCK_WIDTH = 80
BLOCK_HEIGHT = 20


# 球拍类
class Rect:
    def __init__(self):
        self.rect_color = color.DARK_BLUE
        self.rect_center_x = WIDTH // 2
        self.rect_center_y = RECT_HEIGHT // 2
        self.rect_move_x = 45

    def drawRect(self):
        draw_rectangle_filled(self.rect_center_x, self.rect_center_y, RECT_WIDTH,
                              RECT_HEIGHT, self.rect_color)

    # 球拍的移动
    def moveRect(self):
        # 球拍的y坐标被固定，只考虑是否和窗体左右边缘碰撞
        if self.rect_center_x <= RECT_WIDTH // 2:
            self.rect_center_x = RECT_WIDTH // 2
        if self.rect_center_x >= WIDTH - RECT_WIDTH // 2:
            self.rect_center_x = WIDTH - RECT_WIDTH // 2


# 小球类
class Ball:
    def __init__(self):
        self.ball_color = color.RED
        self.ball_speed_x = random.choice([-5, -4, -3, 3, 4, 5])
        self.ball_speed_y = random.choice([3, 4, 5])
        self.ball_radius = 10
        self.ball_center_x = WIDTH // 2
        self.ball_center_y = RECT_HEIGHT + self.ball_radius
        self.rect_x = WIDTH // 2
        self.rect_y = RECT_HEIGHT // 2
        self.over = False

    def drawBall(self):
        draw_circle_filled(self.ball_center_x, self.ball_center_y, self.ball_radius,
                           self.ball_color)

    # 小球的移动
    def moveBall(self):
        # 小球出界
        if self.ball_center_y <= 0:
            self.over = True

        # 小球有没有和窗体左右边缘碰撞
        if self.ball_radius <= self.ball_center_y <= HEIGHT - self.ball_radius:
            if self.ball_center_x <= self.ball_radius or \
                    self.ball_center_x >= WIDTH - self.ball_radius:
                self.ball_speed_x = -self.ball_speed_x

        # 小球有没有和球拍边缘和上边缘碰撞
        elif self.ball_radius <= self.ball_center_x <= WIDTH - self.ball_radius:
            if self.ball_center_y >= HEIGHT - self.ball_radius:  # 上边缘碰撞
                self.ball_speed_y = -self.ball_speed_y
            elif self.ball_center_y <= self.ball_radius + RECT_HEIGHT:  # 球拍边缘碰撞
                if self.rect_x <= self.ball_center_x <= self.rect_x + RECT_WIDTH:
                    self.ball_speed_y = -self.ball_speed_y
                else:
                    self.over = True

        self.ball_center_x += self.ball_speed_x
        self.ball_center_y += self.ball_speed_y


class Brick:
    def __init__(self, center_x, center_y):
        self.color = (139, 126, 102)
        self.center_x = center_x
        self.center_y = center_y
        self.height = BLOCK_HEIGHT
        self.width = BLOCK_WIDTH

    def drawBrick(self):
        draw_rectangle_filled(self.center_x, self.center_y,
                              self.width, self.height, self.color)


# 游戏窗口类
class Game(Window):
    def __init__(self, width, height, title):
        super().__init__(width, height, title)
        self.bg_color = color.SKY_BLUE
        self.ball = Ball()
        self.rect = Rect()
        self.total_time = 0
        self.last_time = 0
        self.game_over = False
        self.win = False
        self.score = 0
        set_background_color(self.bg_color)
        self.block_list = []
        for y in range(570, 209, -50):
            for x in range(40, 761, 144):
                self.block_list.append(Brick(x, y))

    # 绘制界面
    def on_draw(self):
        start_render()
        draw_text(f"Score: {self.score}", 10, 10, color.ORANGE,
                  font_name=("simhei", "PingFang"), font_size=20)
        self.ball.drawBall()
        self.rect.drawRect()
        for brick in self.block_list:
            brick.drawBrick()

        if self.game_over:  # 失败
            draw_text('Game Over', WIDTH // 2 - 100, HEIGHT // 2,
                      color.RED, font_name=('simhei', 'PingFang'), font_size=40)
        if self.win:  # 获胜
            draw_text('You Win', WIDTH // 2 - 50, HEIGHT // 2 - 50,
                      color.RED, font_name=('simhei', 'PingFang'), font_size=40)

    # 更新界面
    def on_update(self, delta_time):
        if self.ball.over:  # 失败
            self.game_over = True

        if self.game_over or self.win:
            return

        self.total_time += delta_time
        if int(self.total_time) % 500 == 0 and int(self.last_time) != int(self.total_time):
            if self.ball.ball_speed_x >= 0:
                self.ball.ball_speed_x += 2
            else:
                self.ball.ball_speed_x -= 2

            if self.ball.ball_speed_y >= 0:
                self.ball.ball_speed_y += 2
            else:
                self.ball.ball_speed_y -= 2
            self.last_time = self.total_time

        self.ball.rect_x = self.rect.rect_center_x - RECT_WIDTH // 2
        self.ball.rect_y = self.rect.rect_center_y + RECT_HEIGHT // 2
        self.ball.moveBall()

    # 按键触发事件
    def on_key_press(self, symbol, modifiers):
        if symbol == key.RIGHT:
            self.rect.rect_center_x += self.rect.rect_move_x
            self.rect.moveRect()

        elif symbol == key.LEFT:
            self.rect.rect_center_x -= self.rect.rect_move_x
            self.rect.moveRect()


# 开始游戏
def run_game():
    Game(WIDTH, HEIGHT, TITLE)
    run()


if __name__ == "__main__":
    run_game()



from arcade import color, key
from arcade import *
import random
import time
import sys

SC_WIDTH = 1000
SC_HEIGHT = 700
SC_TITLE = "模拟新冠肺炎的传染"


# 模拟的学生
class Students:
    def __init__(self):
        self.color = color.GREEN
        self.radius = 12
        self.AxisRange = (0, 0, 0, 0)
        self.classroomNumber = 1
        self.x = 0
        self.y = 0
        self.move_y = random.random() + 0.1
        self.move_x = random.random() + 0.1
        self.isInfected = False
        self.back = False
        self.counts = 0

    def setup(self):
        draw_circle_filled(self.x, self.y, self.radius, self.color)
        if self.y >= 500 or self.y <= 200:
            if self.x - self.radius <= self.AxisRange[0]:
                self.x = self.AxisRange[0] + self.radius
            if self.x + self.radius >= self.AxisRange[1]:
                self.x = self.AxisRange[1] - self.radius

    def MoveInHallway(self):
        self.x += self.move_x
        self.y += self.move_y
        draw_circle_filled(self.x, self.y, self.radius, self.color)
        if self.x <= self.radius or self.x >= SC_WIDTH - self.radius:
            self.move_x = -self.move_x

        if self.y <= self.radius or self.y >= SC_HEIGHT - self.radius:
            self.move_y = -self.move_y

        if 200 + self.radius <= self.y <= 500 - self.radius:
            if self.y == 200 + self.radius or self.y == 500 + self.radius:
                if 184 <= self.x <= 204 or 388 <= self.x <= 408 or 592 <= self.x <= 612 or \
                        796 <= self.x <= 816:
                    self.move_x = -self.move_x
                else:
                    self.x += self.move_x
                    self.y += self.move_y
            else:
                self.x += self.move_x
                self.y += self.move_y

        if self.y > 500 - self.radius or self.y < 200 + self.radius:
            if self.radius <= self.x <= 184 - self.radius or \
                    204 + self.radius <= self.x <= 388 - self.radius or \
                    408 + self.radius <= self.x <= 592 - self.radius or \
                    612 + self.radius <= self.x <= 796 - self.radius or \
                    816 + self.radius <= self.x <= 1000 - self.radius:
                self.move_x = -self.move_x
                draw_circle_filled(self.x, self.y, self.radius, self.color)


# 教室
class Classroom:
    def __init__(self):
        self.color = color.WHITE

    def draw(self):
        draw_rectangle_filled(194, 100, 20, 200, self.color)
        draw_rectangle_filled(194, 600, 20, 200, self.color)
        draw_rectangle_filled(398, 100, 20, 200, self.color)
        draw_rectangle_filled(398, 600, 20, 200, self.color)
        draw_rectangle_filled(602, 100, 20, 200, self.color)
        draw_rectangle_filled(602, 600, 20, 200, self.color)
        draw_rectangle_filled(806, 100, 20, 200, self.color)
        draw_rectangle_filled(806, 600, 20, 200, self.color)


class Covid19(Window):
    def __init__(self, width, height, title):
        super().__init__(width, height, title)
        self.total_time = 0
        self.last_time = 0
        self.InfectRate = 0.3
        self.classroom = Classroom()
        self.students = [Students() for _ in range(100)]
        self.Infected = []
        self.Exposed = []
        self.Susceptible = []
        for stu in self.students:
            self.Susceptible.append(stu)

    def on_draw(self):
        start_render()
        set_background_color(color.BLACK)
        self.classroom.draw()
        for stu in self.students:
            stu.setup()

    def on_update(self, delta_time):
        self.total_time += delta_time
        for stu in self.students:
            stu.setup()
            stu.MoveInHallway()
        if int(self.total_time) != int(self.last_time) and int(self.total_time) % 1 == 0:
            self.Infect()
        self.last_time = self.total_time

    def number(self):
        lst = [10 for _ in range(10)]
        for i in range(100):
            while True:
                temp = random.choice([i for i in range(1, 11)])
                lst[temp - 1] -= 1
                if lst[temp - 1] < 0:
                    continue
                else:
                    break

            self.students[i].classroomNumber = temp
            if self.students[i].classroomNumber == 1:
                self.students[i].AxisRange = (0, 184, 500, 700)
            elif self.students[i].classroomNumber == 2:
                self.students[i].AxisRange = (204, 388, 500, 700)
            elif self.students[i].classroomNumber == 3:
                self.students[i].AxisRange = (408, 592, 500, 700)
            elif self.students[i].classroomNumber == 4:
                self.students[i].AxisRange = (612, 796, 500, 700)
            elif self.students[i].classroomNumber == 5:
                self.students[i].AxisRange = (816, 1000, 500, 700)
            elif self.students[i].classroomNumber == 6:
                self.students[i].AxisRange = (0, 184, 0, 200)
            elif self.students[i].classroomNumber == 7:
                self.students[i].AxisRange = (204, 388, 0, 200)
            elif self.students[i].classroomNumber == 8:
                self.students[i].AxisRange = (408, 592, 0, 200)
            elif self.students[i].classroomNumber == 9:
                self.students[i].AxisRange = (612, 796, 0, 200)
            elif self.students[i].classroomNumber == 10:
                self.students[i].AxisRange = (816, 1000, 0, 200)

    def Infect(self):
        if len(self.Susceptible) > 0:
            infected = [random.choice(self.Susceptible)
                        for _ in range(2*(len(self.Exposed) + len(self.Infected)))]
            try:
                for i in infected:
                    self.Susceptible.remove(i)
                    self.Exposed.append(i)
                    i.color = color.ORANGE
                for j in self.Exposed:
                    self.Infected.append(j)
                    self.Exposed.remove(j)
                    j.color = color.RED
            except ValueError:
                pass
        else:
            time.sleep(3)
            sys.exit()

    def on_key_press(self, symbol, modifiers):
        if symbol == key.KEY_1:
            self.number()
            temp_list = []
            for stu in self.students:
                temp_list.append(stu)
            infected_student = random.choice(temp_list)
            infected_student.color = color.ORANGE
            self.Exposed.append(infected_student)
            self.Susceptible.remove(infected_student)
            for stu in self.students:
                stu.x = random.randint(stu.AxisRange[0], stu.AxisRange[1])
                stu.y = random.randint(stu.AxisRange[2], stu.AxisRange[3])
                stu.setup()
                draw_circle_filled(stu.x, stu.y, stu.radius, stu.color)


def main():
    Covid19(SC_WIDTH, SC_HEIGHT, SC_TITLE)
    run()


if __name__ == "__main__":
    main()



import sys
from PySide2.QtWidgets import*
from PySide2.QtCore import*
from PySide2.QtGui import*
from PySide2.QtMultimedia import QSound


class Fix:
    def analyse_code(self, contents: str) -> tuple:
        lines = contents.splitlines(keepends=True)
        # locate pyside_package_dir line index
        pkg_dir = 'os.path.abspath(os.path.dirname(__file__))'
        for index, line in enumerate(lines):
            if pkg_dir in line:
                break
        return lines, index, ' ' * (len(line.rstrip()) - len(line.strip()))

    def insert_lines(self, contents: str) -> list:
        lines, position, indentation = self.analyse_code(contents)
        new_lines = (
            lines[position] + '\n' +
            indentation + '# add platforms plugin to PATH\n' +
            indentation + 'platforms_dir = ' +
            'os.path.join(pyside_package_dir, "plugins", "platforms")\n' +
            indentation +
            'os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = platforms_dir\n')
        lines[position] = new_lines
        return lines

    def add_plugins_to_PATH(self, file_path):
        with open(file_path, encoding='utf-8') as f:
            contents = f.read()
        lines = self.insert_lines(contents)
        code = ''.join(lines)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(code)

    def start_fix(self):
        import PySide2
        PySide2_init_module = PySide2.__file__
        self.add_plugins_to_PATH(PySide2_init_module)


fix = Fix()
fix.start_fix()


class Tray(QSystemTrayIcon):
    def __init__(self):
        super().__init__()
        self.setup()
        self.show()

    def setup(self):
        self.index = 0
        self.tips = ["我叫编程猫", "很高兴认识你", "很开心能一起学习Python"]
        self.icon = QIcon("images_cat/sit.png")
        self.setIcon(self.icon)
        self.setToolTip("使用鼠标点击我吧")
        self.activated.connect(self.process)
        self.sound1 = QSound("images_cat/sound1.wav")
        self.sound2 = QSound("images_cat/sound2.wav")
        self.icon2 = QIcon("images_cat/stand.png")
        self.timer = QTimer()
        self.icons = [f"images_cat/{i}.png" for i in range(0, 5)]
        self.icon_index = 0
        self.change_icon()
    
    def process(self, key):
        if key == self.Trigger:
            self.sound1.play()
            self.setIcon(self.icon)
        elif key == self.Context or key == self.MiddleClick:
            self.sound2.play()
            self.setIcon(self.icon2)
        self.setToolTip(self.tips[self.index % 3])
        self.index += 1

    def change_icon(self):
        self.setIcon(QIcon(self.icons[self.icon_index % 5]))
        self.icon_index += 1
        self.timer.singleShot(100, self.change_icon)


def main():
    app = QApplication(sys.argv)
    tray = Tray()
    sys.exit(app.exec_())


if __name__ == "__main__":
    main()
